@@ -38,6 +38,7 @@
 
 #include <dbus/dbus-internals.h>
 #include <dbus/dbus-list.h>
+#include <dbus/dbus-sysdeps-unix.h>
 #include "dir-watch.h"
 
 #define MAX_DIRS_TO_WATCH 128
@@ -73,12 +74,12 @@ _handle_kqueue_watch (DBusWatch *watch, unsigned int flags, void *data)
     {
       kq = -1;
       if (watch != NULL)
-	{
-	  _dbus_loop_remove_watch (loop, watch);
+        {
+          _dbus_loop_remove_watch (loop, watch);
           _dbus_watch_invalidate (watch);
           _dbus_watch_unref (watch);
-	  watch = NULL;
-	}
+          watch = NULL;
+        }
       pid = getpid ();
       _dbus_verbose ("Sending SIGHUP signal since kqueue has been closed\n");
       (void) kill (pid, SIGHUP);
@@ -87,11 +88,49 @@ _handle_kqueue_watch (DBusWatch *watch, unsigned int flags, void *data)
   return TRUE;
 }
 
+static void _shutdown_kqueue (void *data)
+{
+  int i;
+
+  if (kq < 0)
+    return;
+
+  for (i = 0; i < MAX_DIRS_TO_WATCH; i++)
+    {
+      if (fds[i] >= 0)
+        {
+          close (fds[i]);
+          fds[i] = -1;
+        }
+      if (dirs[i] != NULL)
+        {
+          /* dbus_free() is necessary to pass memleak check */
+          dbus_free (dirs[i]);
+          dirs[i] = NULL;
+        }
+    }
+
+  if (loop)
+    {
+      _dbus_loop_remove_watch (loop, watch);
+      _dbus_loop_unref (loop);
+      loop = NULL;
+    }
+
+  if (watch)
+    {
+      _dbus_watch_invalidate (watch);
+      _dbus_watch_unref (watch);
+      watch = NULL;
+    }
+
+    close (kq);
+    kq = -1;
+}
+
 static int
 _init_kqueue (BusContext *context)
 {
-  int ret = 0;
-
   if (kq < 0)
     {
 
@@ -99,38 +138,61 @@ _init_kqueue (BusContext *context)
       if (kq < 0)
         {
           _dbus_warn ("Cannot create kqueue; error '%s'\n", _dbus_strerror (errno));
-	  goto out;
-	}
+          goto out;
+        }
 
-        loop = bus_context_get_loop (context);
+      loop = bus_context_get_loop (context);
+      _dbus_loop_ref (loop);
 
-        watch = _dbus_watch_new (kq, DBUS_WATCH_READABLE, TRUE,
-                                 _handle_kqueue_watch, NULL, NULL);
+      watch = _dbus_watch_new (kq, DBUS_WATCH_READABLE, TRUE,
+                               _handle_kqueue_watch, NULL, NULL);
 
-	if (watch == NULL)
-          {
-            _dbus_warn ("Unable to create kqueue watch\n");
-	    close (kq);
-	    kq = -1;
-	    goto out;
-	  }
+      if (watch == NULL)
+        {
+          _dbus_warn ("Unable to create kqueue watch\n");
+          goto out1;
+        }
 
-	if (!_dbus_loop_add_watch (loop, watch))
-          {
-            _dbus_warn ("Unable to add reload watch to main loop");
-	    _dbus_watch_invalidate (watch);
-	    _dbus_watch_unref (watch);
-	    watch = NULL;
-	    close (kq);
-	    kq = -1;
-            goto out;
-	  }
+      if (!_dbus_loop_add_watch (loop, watch))
+        {
+          _dbus_warn ("Unable to add reload watch to main loop");
+          goto out2;
+        }
+
+      if (!_dbus_register_shutdown_func (_shutdown_kqueue, NULL))
+        {
+          _dbus_warn ("Unable to register shutdown function");
+          goto out3;
+        }
     }
 
-  ret = 1;
+  return 1;
+
+out3:
+  _dbus_loop_remove_watch (loop, watch);
+
+out2:
+  if (watch)
+    {
+      _dbus_watch_invalidate (watch);
+      _dbus_watch_unref (watch);
+      watch = NULL;
+    }
+
+out1:
+  if (kq >= 0)
+    {
+      close (kq);
+      kq = -1;
+    }
+  if (loop)
+    {
+      _dbus_loop_unref (loop);
+      loop = NULL;
+    }
 
 out:
-  return ret;
+  return 0;
 }
 
 void
@@ -139,8 +201,11 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
   int new_fds[MAX_DIRS_TO_WATCH];
   char *new_dirs[MAX_DIRS_TO_WATCH];
   DBusList *link;
-  int i, j, f, fd;
+  int i, j, fd;
   struct kevent ev;
+#ifdef O_CLOEXEC
+  dbus_bool_t cloexec_done = 0;
+#endif
 
   if (!_init_kqueue (context))
     goto out;
@@ -169,12 +234,12 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
           if (dirs[j] && strcmp (new_dirs[i], dirs[j]) == 0)
             {
               new_fds[i] = fds[j];
-	      new_dirs[i] = dirs[j];
-	      fds[j] = -1;
-	      dirs[j] = NULL;
-	      break;
-	    }
-	}
+              new_dirs[i] = dirs[j];
+              fds[j] = -1;
+              dirs[j] = NULL;
+              break;
+            }
+        }
     }
 
   /* Any directory we find in "fds" with a nonzero fd must
@@ -185,10 +250,10 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
       if (fds[j] != -1)
         {
           close (fds[j]);
-	  dbus_free (dirs[j]);
-	  fds[j] = -1;
-	  dirs[j] = NULL;
-	}
+          dbus_free (dirs[j]);
+          fds[j] = -1;
+          dirs[j] = NULL;
+        }
     }
 
   for (i = 0; new_dirs[i]; i++)
@@ -196,9 +261,17 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
       if (new_fds[i] == -1)
         {
           /* FIXME - less lame error handling for failing to add a watch;
-	   * we may need to sleep.
-	   */
-          fd = open (new_dirs[i], O_RDONLY);
+           * we may need to sleep.
+           */
+#ifdef O_CLOEXEC
+          fd = open (new_dirs[i], O_RDONLY | O_CLOEXEC);
+          cloexec_done = (fd >= 0);
+
+          if (fd < 0 && errno == EINVAL)
+#endif
+            {
+              fd = open (new_dirs[i], O_RDONLY);
+            }
           if (fd < 0)
             {
               if (errno != ENOENT)
@@ -213,6 +286,12 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
                   continue;
                 }
             }
+#ifdef O_CLOEXEC
+          if (!cloexec_done)
+#endif
+            {
+              _dbus_fd_set_close_on_exec (fd);
+            }
 
           EV_SET (&ev, fd, EVFILT_VNODE, EV_ADD | EV_ENABLE | EV_CLEAR,
                   NOTE_DELETE | NOTE_EXTEND | NOTE_WRITE | NOTE_RENAME, 0, 0);
@@ -223,18 +302,18 @@ bus_set_watched_dirs (BusContext *context, DBusList **directories)
               goto out;
             }
 
-	  new_fds[i] = fd;
-	  new_dirs[i] = _dbus_strdup (new_dirs[i]);
-	  if (!new_dirs[i])
+          new_fds[i] = fd;
+          new_dirs[i] = _dbus_strdup (new_dirs[i]);
+          if (!new_dirs[i])
             {
               /* FIXME have less lame handling for OOM, we just silently fail to
-	       * watch.  (In reality though, the whole OOM handling in dbus is
-	       * stupid but we won't go into that in this comment =) )
-	       */
+               * watch.  (In reality though, the whole OOM handling in dbus is
+               * stupid but we won't go into that in this comment =) )
+               */
               close (fd);
-	      new_fds[i] = -1;
-	    }
-	}
+              new_fds[i] = -1;
+            }
+        }
     }
 
   num_fds = i;
