@@ -124,6 +124,7 @@ static void _dbus_verbose(const char * x,...) {;}
 #  define _dbus_is_verbose() FALSE 
 #endif /* !DBUS_ENABLE_VERBOSE_MODE */
 
+DBUS_PRIVATE_EXPORT
 void _dbus_trace_ref (const char *obj_name,
                       void       *obj,
                       int         old_refcount,
@@ -195,6 +196,9 @@ extern const char *_dbus_return_if_fail_warning_format;
 #define _DBUS_STRUCT_OFFSET(struct_type, member)	\
     ((intptr_t) ((unsigned char*) &((struct_type*) 0)->member))
 
+#define _DBUS_ALIGNOF(type) \
+    (_DBUS_STRUCT_OFFSET (struct { char _1; type _2; }, _2))
+
 #ifdef DBUS_DISABLE_CHECKS
 /* this is an assert and not an error, but in the typical --disable-checks case (you're trying
  * to really minimize code size), disabling these assertions makes sense.
