@@ -17,7 +17,7 @@
  * 
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
+ * Foundation, Inc., 51 Franklin Street, Fifth Floor, Boston, MA  02110-1301  USA
  *
  */
 #ifndef DBUS_TIMEOUT_H
@@ -40,14 +40,18 @@ typedef struct DBusTimeoutList DBusTimeoutList;
 /** function to run when the timeout is handled */
 typedef dbus_bool_t (* DBusTimeoutHandler) (void *data);
 
+DBUS_PRIVATE_EXPORT
 DBusTimeout* _dbus_timeout_new          (int                 interval,
                                          DBusTimeoutHandler  handler,
                                          void               *data,
                                          DBusFreeFunction    free_data_function);
 DBusTimeout* _dbus_timeout_ref          (DBusTimeout        *timeout);
+DBUS_PRIVATE_EXPORT
 void         _dbus_timeout_unref        (DBusTimeout        *timeout);
+DBUS_PRIVATE_EXPORT
 void         _dbus_timeout_set_interval (DBusTimeout        *timeout,
                                          int                 interval);
+DBUS_PRIVATE_EXPORT
 void         _dbus_timeout_set_enabled  (DBusTimeout        *timeout,
                                          dbus_bool_t         enabled);
 
